@@ -126,7 +126,7 @@ class GraphAdminClient:
         self._token = content['access_token']
         self._log(logging.INFO, "token has been refreshed")
 
-    async def _request(self, method, url, headers: dict = None, data: dict = None, expected_statuses=None):
+    async def _request(self, method, url, headers: dict = None, data: dict or str = None, expected_statuses=None):
         if not self._session:
             self._session = aiohttp.ClientSession()
         if not expected_statuses:
@@ -456,12 +456,26 @@ class GraphAdminClient:
         supported_drive_resources = [USERS, SITES, GROUPS]
         if resource not in supported_drive_resources:
             raise GraphClientException(
-                f"getting drive file only available for the resources: {supported_drive_resources}")
+                f"getting drive file content only available for the resources: {supported_drive_resources}")
         url = self._build_url(V1_EP, [(resource, id), (DRIVE, None), ("/items", drive_item_id), ("/content", None)])
         res, status = await self._request("GET", url, kwargs["_req_headers"],
                                           expected_statuses=kwargs.get("expected_statuses"))
         return res, status
 
+    @authorized
+    async def get_drive_item(self, drive_id: str, item_id: str, **kwargs):
+        url = self._build_url(V1_EP, [(DRIVES, drive_id), ("/items", item_id)])
+        res, status = await self._request("GET", url, kwargs["_req_headers"],
+                                          expected_statuses=kwargs.get("expected_statuses"))
+        return res, status
+
+    @authorized
+    async def get_user_drive_item(self, user_id: str, item_id: str, **kwargs):
+        url = self._build_url(V1_EP, [(USERS, user_id), (DRIVE, None), ("/items", item_id)])
+        res, status = await self._request("GET", url, kwargs["_req_headers"],
+                                          expected_statuses=kwargs.get("expected_statuses"))
+        return res, status
+
     @authorized
     async def list_sites_bulk(self, **kwargs):
         url = self._build_url(V1_EP, [(SITES, None)], **kwargs)
@@ -525,3 +539,45 @@ class GraphAdminClient:
         url = self._build_url(V1_EP, [(TEAMS, team_id), (CHANNELS, channel_id)], **kwargs)
         return await self._request("GET", url, kwargs["_req_headers"],
                                    expected_statuses=kwargs.get("expected_statuses"))
+
+    @authorized
+    async def add_extension_to_message(self, user_id, message_id, extension_name, extension_data=None, **kwargs):
+        url = self._build_url(V1_EP, [(USERS, user_id), (MAILS, message_id), (EXTENSIONS, None)], **kwargs)
+        data = {
+            "@odata.type": "microsoft.graph.openTypeExtension",
+            "extensionName": extension_name
+        }
+        data.update(extension_data or {})
+        data = json.dumps(data)
+        return await self._request(
+            "POST", url, kwargs["_req_headers"], expected_statuses=kwargs.get("expected_statuses"), data=data)
+
+    @authorized
+    async def delete_extension_from_message(self, user_id, message_id, extension_name, **kwargs):
+        url = self._build_url(V1_EP, [(USERS, user_id), (MAILS, message_id), (EXTENSIONS, extension_name)], **kwargs)
+        return await self._request(
+            "DELETE", url, kwargs["_req_headers"], expected_statuses=kwargs.get("expected_statuses"))
+
+    @authorized
+    async def list_recent_files(self, resource: str, resource_id: str, **kwargs):
+        supported_drive_resources = [USERS, SITES, GROUPS]
+        if resource not in supported_drive_resources:
+            raise GraphClientException(
+                f"list recent files only available for the resources: {supported_drive_resources}")
+        url = self._build_url(V1_EP, [(resource, resource_id), (DRIVE, None), ("/recent", None)], **kwargs)
+        res, status = await self._request(
+            "GET", url, kwargs["_req_headers"], expected_statuses=kwargs.get("expected_statuses"))
+
+        next_url = res.get(NEXT_KEY)
+        delta_url = res.get(DELTA_KEY)
+        for drive_item in res["value"]:
+            yield drive_item
+        while next_url:
+            res, status = await self.list_more(next_url, **kwargs)
+            next_url = res.get(NEXT_KEY)
+            delta_url = res.get(DELTA_KEY)
+            for drive_item in res["value"]:
+                yield drive_item
+        if not delta_url:
+            raise GraphClientException("missing deltaLink after iterating through all recent drive items")
+        yield delta_url
