@@ -34,9 +34,12 @@ class TestClient(asynctest.TestCase):
     _one_drive_token = None
     _mail_start_time = None
     _mail_end_time = None
+    _expected_count = None
     _mail_to_users = None
     _mail_from_user = None
     _one_drive_file_id = None
+    _remote_drive_id = None
+    _remote_drive_item_id = None
 
     def setUp(self):
         pass
@@ -64,10 +67,13 @@ class TestClient(asynctest.TestCase):
         cls._resource_data_in_subscription_app_secret = details["resource_data_in_subscription_app_secret"]
         cls._mail_start_time = details["mail_start_time"]
         cls._mail_end_time = details["mail_end_time"]
+        cls._expected_count = details["expected_mails_count"]
         cls._mail_to_users = details["mail_to_users"]
         cls._mail_from_user = details["mail_from_user"]
         cls._total_users_count = 25
         cls._bulk_size = 10
+        cls._remote_drive_id = details["remote_drive_id"]
+        cls._remote_drive_item_id = details["remote_drive_item_id"]
 
         token_params = {
             'grant_type': 'client_credentials',
@@ -261,7 +267,7 @@ class TestClient(asynctest.TestCase):
         i = self.get_instance()
 
         q = ODataQuery()
-        q.top = 6
+        q.top = 1
         start = TestClient._mail_start_time
         end = TestClient._mail_end_time
         constrains = [Constrain("receivedDateTime", LogicalOperator.GT, start),
@@ -286,6 +292,7 @@ class TestClient(asynctest.TestCase):
         q.top = 6
         start = TestClient._mail_start_time
         end = TestClient._mail_end_time
+        expected_count = TestClient._expected_count
         constrains = [Constrain("receivedDateTime", LogicalOperator.GT, start),
                       Constrain("receivedDateTime", LogicalOperator.LT, end)]
         q.filter = Filter(constrains, LogicalConnector.AND)
@@ -294,7 +301,7 @@ class TestClient(asynctest.TestCase):
         async for mail in i.list_all_user_mails(TestClient._user_id, token=TestClient._token, odata_query=q):
             mails.append(mail)
 
-        self.assertEqual(9, len(mails))
+        self.assertEqual(expected_count, len(mails))
 
     async def test_list_all_user_mails_filter_mail_address_ne(self):
         i = self.get_instance()
@@ -379,6 +386,43 @@ class TestClient(asynctest.TestCase):
         self.assertEqual(status, HTTPStatus.OK)
         self.assertEqual(bytes, type(mail))
 
+    async def test_add_extension_to_message_and_delete_it(self):
+        i = self.get_instance()
+
+        extension_name = "ScannedByBitDam"
+        extension_data = {
+            "companyName": "Datto",
+            "verdict": "malicious",
+            "scanId": "testing_open_extension_scan_id"
+        }
+
+        mail, status = await i.add_extension_to_message(
+            TestClient._user_id, TestClient._valid_message_id, extension_name=extension_name,
+            extension_data=extension_data, token=TestClient._token)
+
+        self.assertEqual(status, HTTPStatus.CREATED)
+
+        odata_query = ODataQuery()
+        odata_query.top = TestClient._bulk_size
+        odata_query.filter = Filter([Constrain(attribute="Extensions", logical_operator=LogicalOperator.ANY_EQ, value="ScannedByBitDam", inner_attribute="id")])
+
+        res, status = await i.list_user_mails_bulk(
+            TestClient._user_id, token=TestClient._token, odata_query=odata_query)
+        self.assertEqual(status, HTTPStatus.OK)
+        current_with_extensions = len(res["value"])
+        self.assertTrue(current_with_extensions > 0)
+
+        mail, status = await i.delete_extension_from_message(
+            TestClient._user_id, TestClient._valid_message_id, extension_name=extension_name,
+            token=TestClient._token)
+        self.assertEqual(status, HTTPStatus.NO_CONTENT)
+
+        res, status = await i.list_user_mails_bulk(
+            TestClient._user_id, token=TestClient._token, odata_query=odata_query)
+        self.assertEqual(status, HTTPStatus.OK)
+        after_delete_with_extensions = len(res["value"])
+        self.assertEqual(after_delete_with_extensions, current_with_extensions - 1)
+
     async def test_send_mail_basic_exceptions(self):
         i = self.get_instance()
         mail = {}
@@ -452,6 +496,23 @@ class TestClient(asynctest.TestCase):
 
         self.assertIsNotNone(delta_url)
 
+    async def test_list_recent_drive_files(self):
+        i = self.get_instance()
+        odata_query = ODataQuery()
+        odata_query.top = 10
+        odata_query.order_by = OrderBy("lastModifiedDateTime", Order.desc)
+        wanted = 20
+        current = 0
+        recent_drive_items = []
+        async for drive_item in i.list_recent_files(USERS, TestClient._user_id, token=TestClient._token, odata_query=odata_query):
+            if type(drive_item) == dict:
+                current += 1
+                recent_drive_items.append(drive_item)
+            if current == wanted:
+                break
+
+        self.assertEqual(len(recent_drive_items), wanted)
+
     async def test_get_latest_delta_url_site_drive(self):
         i = self.get_instance()
 
@@ -475,6 +536,26 @@ class TestClient(asynctest.TestCase):
         self.assertIsNotNone(drive_item_content)
         self.assertEqual(type(drive_item_content), bytes)
 
+    async def test_get_drive_item(self):
+        i = self.get_instance()
+
+        drive_item, status = await i.get_drive_item(
+            TestClient._remote_drive_id, TestClient._remote_drive_item_id, token=TestClient._one_drive_token)
+
+        self.assertEqual(status, HTTPStatus.OK)
+        self.assertIsNotNone(drive_item)
+        self.assertEqual(type(drive_item), dict)
+
+    async def test_get_user_drive_item(self):
+        i = self.get_instance()
+
+        drive_item, status = await i.get_user_drive_item(
+            TestClient._user_id, TestClient._remote_drive_item_id, token=TestClient._one_drive_token)
+
+        self.assertEqual(status, HTTPStatus.OK)
+        self.assertIsNotNone(drive_item)
+        self.assertEqual(type(drive_item), dict)
+
     async def test_list_some_user_drive_content_using_iterator(self):
         i = self.get_instance()
 
