@@ -19,8 +19,10 @@ SENDMAIL = "/sendmail"
 GROUPS = "/groups"
 SITES = "/sites"
 DRIVE = "/drive"
+DRIVES = "/drives"
 TEAMS = "/teams"
 CHANNELS = "/channels"
+EXTENSIONS = "/extensions"
 
 # next_key
 NEXT_KEY = "@odata.nextLink"
